@@ -13,7 +13,7 @@ sub component_base_class { 'DBIx::Class' }
 # i.e. first release of 0.XX *must* be 0.XX000. This avoids fBSD ports
 # brain damage and presumably various other packaging systems too
 
-$VERSION = '0.06003';
+$VERSION = '0.07000';
 
 sub MODIFY_CODE_ATTRIBUTES {
     my ($class,$code,@attrs) = @_;
@@ -87,7 +87,7 @@ Then you can use these classes in your application's code:
   # Create a result set to search for artists.
   # This does not query the DB.
   my $johns_rs = $schema->resultset('Artist')->search(
-    # Build your WHERE using an L<SQL::Abstract> structure:
+    # Build your WHERE using an SQL::Abstract structure:
     { name => { like => 'John%' } }
   );
 
@@ -180,7 +180,9 @@ andyg: Andy Grundman <andy@hybridized.org>
 
 ank: Andres Kievsky
 
-blblack: Brandon Black
+blblack: Brandon L. Black <blblack@gmail.com>
+
+bluefeet: Aran Deltac <bluefeet@cpan.org>
 
 LTJake: Brian Cassidy <bricas@cpan.org>
 
@@ -204,7 +206,9 @@ castaway: Jess Robinson
 
 quicksilver: Jules Bean
 
-jguenther: Justin Guenther <guentherj@agr.gc.ca>
+jguenther: Justin Guenther <jguenther@cpan.org>
+
+captainL: Luke Saunders <luke.saunders@gmail.com>
 
 draven: Marcus Ramberg <mramberg@cpan.org>
 
@@ -222,12 +226,12 @@ scotty: Scotty Allen <scotty@scottyallen.com>
 
 sszabo: Stephan Szabo <sszabo@bigpanda.com>
 
-captainL: Luke Saunders <luke.saunders@gmail.com>
-
 Todd Lipcon
 
 wdh: Will Hawes
 
+gphat: Cory G Watson <gphat@cpan.org>
+
 =head1 LICENSE
 
 You may distribute this code under the same terms as Perl itself.
